@@ -1,5 +1,5 @@
-#ifndef __RSQLITE_MY_UTILS__
-#define __RSQLITE_MY_UTILS__
+#ifndef __RMARIADB_MY_UTILS__
+#define __RMARIADB_MY_UTILS__
 
 // Generic data frame utils ----------------------------------------------------
 
@@ -12,4 +12,6 @@ List df_create(const std::vector<MariaFieldType>& types,
                const std::vector<std::string>& names,
                int n);
 
+int days_from_civil(int y, int m, int d);
+
 #endif
